"""
PONGsim (Python Overlay Network Graphical Simulator)
Utility for averaging data from several simulator runs

Version: 2.2

Nicklas Beijar, nbeijar@netlab.tkk.fi
Networking Laboratory, Helsinki University of Technology

######################################################################

PONGsim (Python Overlay Network Graphical Simulator)

Copyright (c) 2005-2007 Nicklas Beijar

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

######################################################################

To do:
- PyDoc compatible comments

"""


import os, string

differingallowed = ["file", "runoutput"]

def getfiles(path):
	scenarios = {}
	for filename in os.listdir(path):
		if string.find(filename, "average") == -1:
			fields = string.splitfields(filename, " ")
			if string.find(filename, ".output.txt") > -1:
				scenario = string.joinfields(fields[:-1], " ")
				if not scenarios.has_key(scenario): 
					scenarios[scenario] = []
				scenarios[scenario].append(path + filename)
	return scenarios

def readfile(filename):
	data = {}
	fp = open(filename, "r")
	while 1:
		line = fp.readline()
		if not line: break
		line = line[:-1]
		fields = string.splitfields(line, ";")
		data[fields[0]] = fields[1:]
	fp.close()
	return data

def getparameters(filecontents):
	params = []
	for data in filecontents:
		for key in data.keys():
			if key not in params: params.append(key)
	return params

def getvalues(filecontents, parameter):
	values = []
	for data in filecontents:
		if data.has_key(parameter):
			values.append(data[parameter])
	return values

def average(values):
	defined = True
	result = []
	for component in range(len(values[0])):
		sum = 0.0
		count = 0
		for value in values:
			try:
				sum += float(value[component])
			except ValueError:
				defined = None
			else:
				count += 1
			if not defined: break
		if not defined: break
		result.append(str(sum/count))
	if defined:
		return result
	return None

def identical(values):
	identical = True
	result = []
	for component in range(len(values[0])):
		firstvalue = values[0][component]
		for value in values:
			if value[component] != firstvalue:
				identical = False
		if identical: result.append(firstvalue)
	if identical:
		return result
	return None

def combinescenario(name, files, outfile):
	filecontents = []
	errors = 0
	print "Processing scenario "+name
	for file in files:
		print "  "+file
		filecontents.append(readfile(file))
	parameters = getparameters(filecontents)
        parameters.sort()
	fp = open(outfile, "w")
	for parameter in parameters:
		values = getvalues(filecontents, parameter)
		avg = average(values)
		if avg == None:
			value = identical(values)
			if (value == None) and (parameter not in differingallowed):
				print "ERROR: Differing non-numerical values:",parameter,"=",values
				avg = None
				errors += 1
			else:
				avg = value
		if avg != None: fp.write(parameter+";"+string.joinfields(avg, ";")+"\n")
	fp.close()
	return errors
	
def main():
	inpath = ".\\output\\"
        outpath = ".\\output\\averages\\"
	errors = 0
	scenarios = getfiles(inpath)
	for scenario in scenarios:
		errors += combinescenario(scenario, scenarios[scenario], outpath+scenario+" average.output.txt")
	if errors > 0:
		print str(errors)+" errors detected"
	else:
		print "No errors detected"
main()
