"""
PONGsim (Python Overlay Network Graphical Simulator)
Pseudo bloom filter module of the PONGsim framework

Version: 1.00

Nicklas Beijar, nbeijar@netlab.tkk.fi
Networking Laboratory, Helsinki University of Technology

######################################################################

PONGsim (Python Overlay Network Graphical Simulator)

Copyright (c) 2005-2007 Nicklas Beijar

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

######################################################################

To do:
- Nothing

"""

import random

####################################################################
###   B l o o m F i l t e r
####################################################################

class BloomFilter:
    
    """A class representing a pseudo Bloom filter.
    
    The filter is only a representation of a real Bloom filter, since it does not provide compression and
    the false positive probability is defined. This allows for controlled experimentation.
    """

    def __init__(self, resources=None, falsepositiveprobability=0.0):
        """Initiate a pseudo Bloom filter and add the given resources to it."""
        self.filter = []
	self.falsepositiveprobability = falsepositiveprobability
        if resources:
            for resource in resources:
                self.filter.append(resource)
        
    def add(self, resource):
        """Add the given resource to this pseudo Bloom filter."""
        self.filter.append(resource)
        
    def check(self, resource):
        """Check if the given resource is included in this pseudo Bloom filter."""
        if random.uniform(0, 100) < self.falsepositiveprobability: return True
        return resource in self.filter

    def __str__(self):
        """Return a string representation of the Bloom filter."""
        return str(self.filter)
