"""
PONGsim (Python Overlay Network Graphical Simulator)
Utility for creating tables from simulator data

Version: 2.0

Nicklas Beijar, nbeijar@netlab.tkk.fi
Networking Laboratory, Helsinki University of Technology

######################################################################

PONGsim (Python Overlay Network Graphical Simulator)

Copyright (c) 2005-2007 Nicklas Beijar

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following
conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

######################################################################

To do:
- PyDoc compatible comments

"""

import os, string

def readfile(filename):
	data = {}
	fp = open(filename, "r")
	while 1:
		line = fp.readline()
		if not line: break
		line = line[:-1]
		fields = string.splitfields(line, ";")
		data[fields[0]] = fields[1:]
	fp.close()
	return data

def getvalue(filename, parameter, field):
    try:
        data = readfile(filename)
    except:
        print "Cannot read file",filename
        return "!FILE_ERROR!"
    try:
        value = data[parameter]
    except KeyError:
        print "Parameter not found:",parameter
        return "!PARAMETER_NOT_FOUND!"
    try:
        value = value[int(field)]
    except KeyError:
        print "Field not found:",field
        value = "!FIELD_NOT_FOUND!"
    return value

def parse(pattern, inpath):
    while string.find(pattern, "[") >= 0:
        start = string.find(pattern, "[")
        mid1 = string.find(pattern, "#", start)
        mid2 = string.find(pattern, "#", mid1+1)
        mid3 = string.find(pattern, "#", mid2+1)
        end = string.find(pattern, "]", mid2)
        if mid3 < 0 or mid3 > end:
            mid3 = None
        if start < 0 or mid1 < 0 or mid2 < 0 or end < 0 or end<=mid2 or mid2<=mid1 or mid1<=start:
            print "Illegal pattern:",pattern
            return "!PATTERN_ERROR!"
        file = pattern[start+1:mid1]
        param = pattern[mid1+1:mid2]
        if mid3:
            field = pattern[mid2+1:mid3]
            default = pattern[mid3+1:end]
            value = getvalue(inpath + file + ".output.txt", param, field)
            try:
                if value[0] == "!": value = default
            except:
                pass
        else:
            field = pattern[mid2+1:end]
            value = getvalue(inpath + file + ".output.txt", param, field)
        pattern = pattern[:start] + value + pattern[end+1:]
    return pattern

def parse_simple(pattern, inpath):
    while string.find(pattern, "[") >= 0:
        start = string.find(pattern, "[")
        mid1 = string.find(pattern, "#", start)
        mid2 = string.find(pattern, "#", mid1+1)
        end = string.find(pattern, "]", mid2)
        if start < 0 or mid1 < 0 or mid2 < 0 or end < 0 or end<=mid2 or mid2<=mid1 or mid1<=start:
            print "Illegal pattern:",pattern
            return "!PATTERN_ERROR!"
        file = pattern[start+1:mid1]
        param = pattern[mid1+1:mid2]
        field = pattern[mid2+1:end]
        value = getvalue(inpath + file + ".output.txt", param, field)
        pattern = pattern[:start] + value + pattern[end+1:]
    return pattern

def maketable(inpath, outpath, outfile, tables, extras, cols, rows, pattern):
        fp = open(outpath+outfile, "w")

        for table in string.splitfields(tables, ";"):
            line = ""
            for extra in string.splitfields(extras, ";"):
                for col in string.splitfields(cols, ";"):
                    line += col + extra + ";"
            fp.write(table+";"+line[:-1]+" \n")
            for row in string.splitfields(rows, ";"):
                line = ""
                for extra in string.splitfields(extras, ";"):
                    for col in string.splitfields(cols, ";"):
                        expression = parse(string.replace(string.replace(string.replace(string.replace(string.replace(pattern, "$C", col), "$R", row), "$T", table), "$P", inpath), "$E", extra), inpath)
                        if expression[0] != "!": 
                            try:
                                expression = str(eval(expression))
                            except:
                                pass # got a string expression
                        line += expression + ";"
                fp.write(row+";"+line[:-1]+" \n")
            fp.write("\n")
        
        fp.close()

def maketable_scalability(inpath, outpath):
        maketable(inpath, outpath, "scalability-msg.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#received-search-msg#4]/[$C_c$R_r1$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "scalability-msg-max.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#received-search-msg#3]/[$C_c$R_r1$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "scalability-delay.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#search-delay#4]*1000.0")
        maketable(inpath, outpath, "scalability-delay-first.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#search-delay-first#4]*1000.0")
        maketable(inpath, outpath, "scalability-delay-last.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#search-delay-last#4]*1000.0")
        maketable(inpath, outpath, "scalability-links.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#searchlinks#0]*[$C_c$R_r1$E average#searchlinks#4]*4000.0")
        maketable(inpath, outpath, "scalability-successrate.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#found-match#4]/[$C_c$R_r1$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "scalability-successrate2.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "3;4;5;6;7;8;9;10;11;12;13;14;15", 
        "[$C_c$R_r1$E average#search-found#4]/[$C_c$R_r1$E average#generated-searches#4]")
        


def maketable_replication(inpath, outpath):
        maketable(inpath, outpath, "replication-msg.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#received-search-msg#4]/[$C_c10_r$R$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "replication-msg-max.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#received-search-msg#3]/[$C_c10_r$R$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "replication-delay.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#search-delay#4]*1000.0")
        maketable(inpath, outpath, "replication-delay-first.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#search-delay-first#4]*1000.0")
        maketable(inpath, outpath, "replication-delay-last.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#search-delay-last#4]*1000.0")
        maketable(inpath, outpath, "replication-links.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#searchlinks#0]*[$C_c10_r$R$E average#searchlinks#4]*4000.0")
        maketable(inpath, outpath, "replication-successrate.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#found-match#4]/[$C_c10_r$R$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "replication-successrate2.txt", "", "_nf;_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5", 
        "[$C_c10_r$R$E average#search-found#4]/[$C_c10_r$R$E average#generated-searches#4]")

def maketable_balance(inpath, outpath):
        maketable(inpath, outpath, "balance-msg.txt", "", "_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#received-search-msg#4]/[$C_p$R_r1$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "balance-msg-max.txt", "", "_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#received-search-msg#3]/[$C_p$R_r1$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "balance-delay.txt", "", "_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#search-delay#4]*1000.0")
        maketable(inpath, outpath, "balance-delay-first.txt", "", "_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#search-delay-first#4]*1000.0")
        maketable(inpath, outpath, "balance-links.txt", "", "_nf;_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#searchlinks#0]*[$C_p$R_r1$E average#searchlinks#4]*4000.0")
        maketable(inpath, outpath, "balance-successrate.txt", "", "_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#found-match#4]/[$C_p$R_r1$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "balance-successrate2.txt", "", "_f", "pic;ipic_sbrw;ipic_hsbrw", "1;2;3;4;5;6", 
        "[$C_p$R_r1$E average#search-found#4]/[$C_p$R_r1$E average#generated-searches#4]")

def maketable_index(inpath, outpath):
        #maketable(inpath, outpath, "index-msg.txt", "", "_f", "flood4;flood8;pic;ipic_sbrw;ipic_hsbrw", "2.5;5;7.5;10;12.5;15;17.5;20", 
        #"[$C_i$R_c10_r0$E average#received-index-msg#4]/[$C_i$R_c10_r0$E average#sent-search-msg#4]")
        maketable(inpath, outpath, "index-msg.txt", "", "_", "2;4;6;8;10", "0.3125;0.625;1.25;2.5;5;10;20;40;80", 
        "([index_nc$C_i$R average#received-index-msg#4]/[index_nc$C_i$R average#sent-search-msg#4]/10.0)/2.0-1.0")

def maketable_g_indexquery(inpath, outpath):
        columns = "floodindex;floodsearch;ri"
        #rows = "1350;2700;5400;10800;21600;43200;86400;172800;345600;691200"
        rows = "691200;345600;172800;86400;43200;21600;10800;5400;2700;1350"
        maketable(inpath, outpath, "indexquery-msg.txt", "", "_", columns, rows, 
        "[$C_n100_d10_i86400_s$R average#received-msg#4]/[$C_n100_d10_i86400_s$R average#generated-searches#4]")
        maketable(inpath, outpath, "indexquery-successrate.txt", "", "_", columns, rows,
        "[$C_n100_d10_i86400_s$R average#search-result-found#4]/[$C_n100_d10_i86400_s$R average#generated-searches#4]")
        maketable(inpath, outpath, "indexquery-delay-first.txt", "", "_", columns, rows,
        "[$C_n100_d10_i86400_s$R average#search-delay-first#4]*1000.0")
        maketable(inpath, outpath, "indexquery-msg-index.txt", "", "_", columns, rows,
        "([$C_n100_d10_i86400_s$R average#received-update#4#0]+[$C_n100_d10_i86400_s$R average#received-ack#4#0]+[$C_n100_d10_i86400_s$R average#received-index-msg#4#0])/[$C_n100_d10_i86400_s$R average#generated-searches#4]")
        maketable(inpath, outpath, "indexquery-msg-search.txt", "", "_", columns, rows,
        "([$C_n100_d10_i86400_s$R average#received-searchquery#4#0]+[$C_n100_d10_i86400_s$R average#received-search-msg#4#0])/[$C_n100_d10_i86400_s$R average#generated-searches#4]")

def maketable_g_degree(inpath, outpath):
        columns = "floodindex;floodsearch;ri"
        rows = "5;10;15;20;25;30;35;40"
        maketable(inpath, outpath, "degree-msg.txt", "", "_", columns, rows, 
        "[$C_n50_d$R_i86400_s86400 average#received-msg#4]/[$C_n50_d$R_i86400_s86400 average#generated-searches#4]")
        maketable(inpath, outpath, "degree-successrate.txt", "", "_", columns, rows,
        "[$C_n50_d$R_i86400_s86400 average#search-result-found#4]/[$C_n50_d$R_i86400_s86400 average#generated-searches#4]")
        maketable(inpath, outpath, "degree-delay-first.txt", "", "_", columns, rows,
        "[$C_n50_d$R_i86400_s86400 average#search-delay-first#4]*1000.0")

def maketable_g_scalability(inpath, outpath):
        columns = "floodindex;floodsearch;ri"
        rows = "10;20;30;40;50;60;70;80;90;100;110;120"
        maketable(inpath, outpath, "scalability-msg.txt", "", "_", columns, rows, 
        "[$C_n$R_d10_i86400_s86400 average#received-msg#4]/[$C_n$R_d10_i86400_s86400 average#generated-searches#4]")
        maketable(inpath, outpath, "scalability-successrate.txt", "", "_", columns, rows,
        "[$C_n$R_d10_i86400_s86400 average#search-result-found#4]/[$C_n$R_d10_i86400_s86400 average#generated-searches#4]")
        maketable(inpath, outpath, "scalability-delay-first.txt", "", "_", columns, rows,
        "[$C_n$R_d10_i86400_s86400 average#search-delay-first#4]*1000.0")
        
inpath = ".\\output\\averages\\"
outpath = ".\\output\\tables\\"
#maketable_scalability(inpath, outpath)
#maketable_replication(inpath, outpath)
#maketable_balance(inpath, outpath)
#maketable_index(inpath, outpath)
maketable_g_indexquery(inpath, outpath)
maketable_g_degree(inpath, outpath)
maketable_g_scalability(inpath, outpath)